@@ -3,33 +3,33 @@ from django.contrib.auth import views as auth_views
 from django.core.urlresolvers import reverse_lazy
 
 from oscar.core.application import Application
-from oscar.apps.catalogue.app import application as catalogue_app
-from oscar.apps.customer.app import application as customer_app
-from oscar.apps.basket.app import application as basket_app
-from oscar.apps.checkout.app import application as checkout_app
-from oscar.apps.promotions.app import application as promotions_app
-from oscar.apps.search.app import application as search_app
-from oscar.apps.offer.app import application as offer_app
-from oscar.apps.dashboard.app import application as dashboard_app
-
 from oscar.apps.customer import forms
+from oscar.core.loading import get_class
 from oscar.views.decorators import login_forbidden
 
 
 class Shop(Application):
     name = None
 
-    catalogue_app = catalogue_app
-    customer_app = customer_app
-    basket_app = basket_app
-    checkout_app = checkout_app
-    promotions_app = promotions_app
-    search_app = search_app
-    dashboard_app = dashboard_app
-    offer_app = offer_app
+    catalogue_app = get_class('catalogue.app', 'application')
+    customer_app = get_class('customer.app', 'application')
+    basket_app = get_class('basket.app', 'application')
+    checkout_app = get_class('checkout.app', 'application')
+    promotions_app = get_class('promotions.app', 'application')
+    search_app = get_class('search.app', 'application')
+    dashboard_app = get_class('dashboard.app', 'application')
+    offer_app = get_class('offer.app', 'application')
 
     def get_urls(self):
-        urlpatterns = patterns('',
+        # After we drop support for Django<1.6 the following line won't be
+        # necessary, and the parameter uidb36 should be replaced with uidb64.
+        # The necessary update should also be done in
+        # oscar/apps/customer/utils.py
+        password_reset_confirm = getattr(
+            auth_views, 'password_reset_confirm_uidb36',
+            auth_views.password_reset_confirm)
+
+        urls = [
             (r'^i18n/', include('django.conf.urls.i18n')),
             (r'^catalogue/', include(self.catalogue_app.urls)),
             (r'^basket/', include(self.basket_app.urls)),
@@ -39,7 +39,7 @@ class Shop(Application):
             (r'^dashboard/', include(self.dashboard_app.urls)),
             (r'^offers/', include(self.offer_app.urls)),
 
-            # Password reset - as we're using Django's default view funtions,
+            # Password reset - as we're using Django's default view functions,
             # we can't namespace these urls as that prevents
             # the reverse function from working.
             url(r'^password-reset/$',
@@ -50,16 +50,18 @@ class Shop(Application):
             url(r'^password-reset/done/$',
                 login_forbidden(auth_views.password_reset_done),
                 name='password-reset-done'),
-            url(r'^password-reset/confirm/(?P<uidb36>[0-9A-Za-z]{1,13})-(?P<token>[0-9A-Za-z]{1,13}-[0-9A-Za-z]{1,20})/$',
-                login_forbidden(auth_views.password_reset_confirm),
-                {'post_reset_redirect': reverse_lazy('password-reset-complete')},
+            url(r'^password-reset/confirm/(?P<uidb36>[0-9A-Za-z]{1,13})-'
+                r'(?P<token>[0-9A-Za-z]{1,13}-[0-9A-Za-z]{1,20})/$',
+                login_forbidden(password_reset_confirm),
+                {'post_reset_redirect':
+                 reverse_lazy('password-reset-complete')},
                 name='password-reset-confirm'),
             url(r'^password-reset/complete/$',
                 login_forbidden(auth_views.password_reset_complete),
                 name='password-reset-complete'),
             (r'', include(self.promotions_app.urls)),
-        )
-        return urlpatterns
+        ]
+        return patterns('', *urls)
 
 
 # 'shop' kept for legacy projects - 'application' is a better name
