@@ -20,4 +20,4 @@ class UserSearch(AbstractUserSearch):
 
 
 # Import receiver functions
-from oscar.apps.analytics.receivers import *
\ No newline at end of file
+from oscar.apps.analytics.receivers import *  # noqa
