@@ -1,42 +1,52 @@
 from itertools import chain
 from datetime import datetime, date
+import logging
+import os
+import warnings
 
 from django.conf import settings
-from django.core.exceptions import ObjectDoesNotExist, ValidationError
+from django.contrib.staticfiles.finders import find
+from django.core.exceptions import ValidationError, ImproperlyConfigured
+from django.core.files.base import File
 from django.core.validators import RegexValidator
 from django.db import models
 from django.db.models import Sum, Count, get_model
 from django.utils.translation import ugettext_lazy as _
+from django.utils.functional import cached_property
 from treebeard.mp_tree import MP_Node
 
 from oscar.core.utils import slugify
-from oscar.core.loading import get_class
-BrowsableProductManager = get_class(
-    'catalogue.managers', 'BrowsableProductManager')
+from oscar.core.loading import get_classes
+
+ProductManager, BrowsableProductManager = get_classes(
+    'catalogue.managers', ['ProductManager', 'BrowsableProductManager'])
 
 
 class AbstractProductClass(models.Model):
     """
-    Defines the options and attributes for a group of products, e.g. Books,
-    DVDs and Toys.
+    Used for defining options and attributes for a subset of products.
+    E.g. Books, DVDs and Toys. A product can only belong to one product class.
+
+    At least one product class must be created when setting up a new
+    Oscar deployment.
 
     Not necessarily equivalent to top-level categories but usually will be.
     """
     name = models.CharField(_('Name'), max_length=128)
     slug = models.SlugField(_('Slug'), max_length=128, unique=True)
 
-    # Some product type don't require shipping (eg digital products) - we use
-    # this field to take some shortcuts in the checkout.
+    #: Some product type don't require shipping (eg digital products) - we use
+    #: this field to take some shortcuts in the checkout.
     requires_shipping = models.BooleanField(_("Requires shipping?"),
                                             default=True)
 
-    # Digital products generally don't require their stock levels to be
-    # tracked.
+    #: Digital products generally don't require their stock levels to be
+    #: tracked.
     track_stock = models.BooleanField(_("Track stock levels?"), default=True)
 
-    # These are the options (set by the user when they add to basket) for this
-    # item class.  For instance, a product class of "SMS message" would always
-    # require a message to be specified before it could be bought.
+    #: These are the options (set by the user when they add to basket) for this
+    #: item class.  For instance, a product class of "SMS message" would always
+    #: require a message to be specified before it could be bought.
     options = models.ManyToManyField('catalogue.Option', blank=True,
                                      verbose_name=_("Options"))
 
@@ -64,7 +74,7 @@ class AbstractCategory(MP_Node):
     name = models.CharField(_('Name'), max_length=255, db_index=True)
     description = models.TextField(_('Description'), blank=True, null=True)
     image = models.ImageField(_('Image'), upload_to='categories', blank=True,
-                              null=True)
+                              null=True, max_length=255)
     slug = models.SlugField(_('Slug'), max_length=255, db_index=True,
                             editable=False)
     full_name = models.CharField(_('Full Name'), max_length=255,
@@ -76,19 +86,33 @@ class AbstractCategory(MP_Node):
     def __unicode__(self):
         return self.full_name
 
+    def update_slug(self, commit=True):
+        """
+        Updates the instance's slug. Use update_children_slugs for updating
+        the rest of the tree.
+        """
+        parent = self.get_parent()
+        slug = slugify(self.name)
+        # If category has a parent, includes the parents slug in this one
+        if parent:
+            self.slug = '%s%s%s' % (
+                parent.slug, self._slug_separator, slug)
+            self.full_name = '%s%s%s' % (
+                parent.full_name, self._full_name_separator, self.name)
+        else:
+            self.slug = slug
+            self.full_name = self.name
+        if commit:
+            self.save()
+
+    def update_children_slugs(self):
+        for child in self.get_children():
+            child.update_slug()
+            child.update_children_slugs()
+
     def save(self, update_slugs=True, *args, **kwargs):
         if update_slugs:
-            parent = self.get_parent()
-            slug = slugify(self.name)
-            # If category has a parent, includes the parents slug in this one
-            if parent:
-                self.slug = '%s%s%s' % (
-                    parent.slug, self._slug_separator, slug)
-                self.full_name = '%s%s%s' % (
-                    parent.full_name, self._full_name_separator, self.name)
-            else:
-                self.slug = slug
-                self.full_name = self.name
+            self.update_slug(commit=False)
 
         # Enforce slug uniqueness here as MySQL can't handle a unique index on
         # the slug field
@@ -103,24 +127,23 @@ class AbstractCategory(MP_Node):
                         'slug': self.slug})
 
         super(AbstractCategory, self).save(*args, **kwargs)
+        self.update_children_slugs()
 
     def move(self, target, pos=None):
         """
         Moves the current node and all its descendants to a new position
         relative to another node.
 
-        See https://tabo.pe/projects/django-treebeard/docs/1.61/api.html#treebeard.models.Node.move
+        See https://tabo.pe/projects/django-treebeard/docs/1.61/api.html#treebeard.models.Node.move  # noqa
         """
         super(AbstractCategory, self).move(target, pos)
 
-        # Update the slugs and full names of all nodes in the new subtree.
         # We need to reload self as 'move' doesn't update the current instance,
         # then we iterate over the subtree and call save which automatically
         # updates slugs.
         reloaded_self = self.__class__.objects.get(pk=self.pk)
-        subtree = self.__class__.get_tree(parent=reloaded_self)
-        for node in subtree:
-            node.save()
+        reloaded_self.update_slug()
+        reloaded_self.update_children_slugs()
 
     def get_ancestors(self, include_self=True):
         ancestors = list(super(AbstractCategory, self).get_ancestors())
@@ -130,8 +153,8 @@ class AbstractCategory(MP_Node):
 
     @models.permalink
     def get_absolute_url(self):
-        return ('catalogue:category', (), {
-                'category_slug': self.slug})
+        return ('catalogue:category', (),
+                {'category_slug': self.slug, 'pk': self.pk})
 
     class Meta:
         abstract = True
@@ -230,25 +253,25 @@ class AbstractProductContributor(models.Model):
 class AbstractProduct(models.Model):
     """
     The base product object
-    """
-    # If an item has no parent, then it is the "canonical" or abstract version
-    # of a product which essentially represents a set of products.  If a
-    # product has a parent then it is a specific version of a catalogue.
-    #
-    # For example, a canonical product would have a title like "Green fleece"
-    # while its children would be "Green fleece - size L".
 
-    # Universal product code
-    upc = models.CharField(_("UPC"), max_length=64, blank=True, null=True,
-                           unique=True,
+    If an item has no parent, then it is the "canonical" or abstract version
+    of a product which essentially represents a set of products.  If a
+    product has a parent then it is a specific version of a catalogue.
+
+    For example, a canonical product would have a title like "Green fleece"
+    while its children would be "Green fleece - size L".
+    """
+    #: Universal product code
+    upc = models.CharField(
+        _("UPC"), max_length=64, blank=True, null=True, unique=True,
         help_text=_("Universal Product Code (UPC) is an identifier for "
                     "a product which is not specific to a particular "
                     " supplier. Eg an ISBN for a book."))
 
     # No canonical product should have a stock record as they cannot be bought.
-    parent = models.ForeignKey('self', null=True, blank=True,
-                               related_name='variants',
-                               verbose_name=_("Parent"),
+    parent = models.ForeignKey(
+        'self', null=True, blank=True, related_name='variants',
+        verbose_name=_("Parent"),
         help_text=_("Only choose a parent product if this is a 'variant' of "
                     "a canonical catalogue.  For example if this is a size "
                     "4 of a particular t-shirt.  Leave blank if this is a "
@@ -260,12 +283,13 @@ class AbstractProduct(models.Model):
     slug = models.SlugField(_('Slug'), max_length=255, unique=False)
     description = models.TextField(_('Description'), blank=True, null=True)
 
-    # Use this field to indicate if the product is inactive or awaiting
-    # approval
+    #: Use this field to indicate if the product is inactive or awaiting
+    #: approval
     status = models.CharField(_('Status'), max_length=128, blank=True,
                               null=True, db_index=True)
     product_class = models.ForeignKey(
         'catalogue.ProductClass', verbose_name=_('Product Class'), null=True,
+        related_name="products",
         help_text=_("""Choose what type of product this is"""))
     attributes = models.ManyToManyField(
         'catalogue.ProductAttribute',
@@ -285,9 +309,8 @@ class AbstractProduct(models.Model):
         verbose_name=_("Related Products"),
         help_text=_("Related items are things like different formats of the "
                     "same book.  Grouping them together allows better linking "
-                    "betwen products on the site."))
+                    "between products on the site."))
 
-    # Recommended products
     recommended_products = models.ManyToManyField(
         'catalogue.Product', through='ProductRecommendation', blank=True,
         verbose_name=_("Recommended Products"))
@@ -308,11 +331,57 @@ class AbstractProduct(models.Model):
         'catalogue.Category', through='ProductCategory',
         verbose_name=_("Categories"))
 
-    is_discountable = models.BooleanField(_("Is Discountable"), default=True)
+    #: Determines if a product may be used in an offer. It is illegal to
+    #: discount some types of product (e.g. ebooks) and this field helps
+    #: merchants from avoiding discounting such products
+    is_discountable = models.BooleanField(
+        _("Is discountable?"), default=True, help_text=(
+            "This flag indicates if this product can be used in an offer "
+            "or not"))
 
-    objects = models.Manager()
+    objects = ProductManager()
     browsable = BrowsableProductManager()
 
+    class Meta:
+        abstract = True
+        ordering = ['-date_created']
+        verbose_name = _('Product')
+        verbose_name_plural = _('Products')
+
+    def __init__(self, *args, **kwargs):
+        super(AbstractProduct, self).__init__(*args, **kwargs)
+        self.attr = ProductAttributesContainer(product=self)
+
+    def __unicode__(self):
+        if self.is_variant:
+            return u"%s (%s)" % (self.get_title(), self.attribute_summary)
+        return self.get_title()
+
+    @models.permalink
+    def get_absolute_url(self):
+        u"""Return a product's absolute url"""
+        return ('catalogue:detail', (), {
+            'product_slug': self.slug,
+            'pk': self.id})
+
+    def save(self, *args, **kwargs):
+        if self.is_top_level and not self.title:
+            raise ValidationError(_("Canonical products must have a title"))
+        if not self.slug:
+            self.slug = slugify(self.get_title())
+
+        # Allow attribute validation to be skipped.  This is required when
+        # saving a parent product which belongs to a product class with
+        # required attributes.
+        if kwargs.pop('validate_attributes', True):
+            self.attr.validate_attributes()
+
+        # Save product
+        super(AbstractProduct, self).save(*args, **kwargs)
+
+        # Finally, save attributes
+        self.attr.save()
+
     # Properties
 
     @property
@@ -330,15 +399,12 @@ class AbstractProduct(models.Model):
         """
         return self.parent_id is None
 
-    @property
+    @cached_property
     def is_group(self):
         """
         Test if this is a top level product and has more than 0 variants
         """
-        # use len() instead of count() in this specific instance
-        # as variants are highly likely to be used after this
-        # which reduces the amount of SQL queries required
-        return self.is_top_level and len(self.variants.all()) > 0
+        return self.is_top_level and self.variants.exists()
 
     @property
     def is_variant(self):
@@ -349,11 +415,64 @@ class AbstractProduct(models.Model):
     def is_shipping_required(self):
         return self.product_class.requires_shipping
 
+    @property
+    def has_stockrecords(self):
+        """
+        Test if this product has any stockrecords
+        """
+        return self.num_stockrecords > 0
+
+    @property
+    def num_stockrecords(self):
+        return self.stockrecords.all().count()
+
+    @property
+    def attribute_summary(self):
+        """
+        Return a string of all of a product's attributes
+        """
+        pairs = []
+        for value in self.attribute_values.select_related().all():
+            pairs.append("%s: %s" % (value.attribute.name,
+                                     value.value))
+        return ", ".join(pairs)
+
+    # Deprecated stockrecord methods
+
+    @property
+    def has_stockrecord(self):
+        """
+        Test if this product has a stock record
+        """
+        warnings.warn(("Product.has_stockrecord is deprecated in favour of "
+                       "using the stockrecord template tag.  It will be "
+                       "removed in v0.8"), DeprecationWarning)
+        return self.num_stockrecords > 0
+
+    @property
+    def stockrecord(self):
+        """
+        Return the stockrecord associated with this product.  For backwards
+        compatibility, this defaults to choosing the first stockrecord found.
+        """
+        # This is the old way of fetching a stockrecord, when they were
+        # one-to-one with a product.
+        warnings.warn(("Product.stockrecord is deprecated in favour of "
+                       "using the stockrecord template tag.  It will be "
+                       "removed in v0.7"), DeprecationWarning)
+        try:
+            return self.stockrecords.all()[0]
+        except IndexError:
+            return None
+
     @property
     def is_available_to_buy(self):
         """
         Test whether this product is available to be purchased
         """
+        warnings.warn(("Product.is_available_to_buy is deprecated in favour "
+                       "of using the stockrecord template tag.  It will be "
+                       "removed in v0.7"), DeprecationWarning)
         if self.is_group:
             # If any one of this product's variants is available, then we treat
             # this product as available.
@@ -365,49 +484,45 @@ class AbstractProduct(models.Model):
             return True
         return self.has_stockrecord and self.stockrecord.is_available_to_buy
 
-    @property
-    def min_variant_price_incl_tax(self):
-        """Return minimum variant price including tax"""
-        return self._min_variant_price('price_incl_tax')
-
-    @property
-    def min_variant_price_excl_tax(self):
-        """Return minimum variant price excluding tax"""
-        return self._min_variant_price('price_excl_tax')
-
-    @property
-    def has_stockrecord(self):
-        """
-        Test if this product has a stock record
-        """
-        try:
-            self.stockrecord
-        except ObjectDoesNotExist:
-            return False
-        else:
-            return True
-
     def is_purchase_permitted(self, user, quantity):
         """
         Test whether this product can be bought by the passed user.
         """
-        if not self.has_stockrecord:
+        warnings.warn(("Product.is_purchase_permitted is deprecated in favour "
+                       "of using a partner strategy.  It will be "
+                       "removed in v0.7"), DeprecationWarning)
+        if not self.has_stockrecords:
             return False, _("No stock available")
-        return self.stockrecord.is_purchase_permitted(user, quantity)
+        return self.stockrecord.is_purchase_permitted(user, quantity, self)
 
-    def add_category_from_breadcrumbs(self, breadcrumb):
-        from oscar.apps.catalogue.categories import create_from_breadcrumbs
-        category = create_from_breadcrumbs(breadcrumb)
+    @property
+    def min_variant_price_incl_tax(self):
+        """
+        Return minimum variant price including tax
+        """
+        return self._min_variant_price('price_incl_tax')
 
-        temp = get_model('catalogue', 'ProductCategory')(
-            category=category, product=self)
-        temp.save()
-    add_category_from_breadcrumbs.alters_data = True
+    @property
+    def min_variant_price_excl_tax(self):
+        """
+        Return minimum variant price excluding tax
+        """
+        return self._min_variant_price('price_excl_tax')
 
-    def attribute_summary(self):
-        u"""Return a string of all of a product's attributes"""
-        return ", ".join([
-            attribute.__unicode__() for attribute in self.attributes.all()])
+    def _min_variant_price(self, property):
+        """
+        Return minimum variant price
+        """
+        prices = []
+        for variant in self.variants.all():
+            if variant.has_stockrecords:
+                prices.append(getattr(variant.stockrecord, property))
+        if not prices:
+            return None
+        prices.sort()
+        return prices[0]
+
+    # Wrappers
 
     def get_title(self):
         """
@@ -417,16 +532,20 @@ class AbstractProduct(models.Model):
         if not title and self.parent_id:
             title = self.parent.title
         return title
+    get_title.short_description = _("Title")
 
     def get_product_class(self):
         """
         Return a product's item class
         """
-        if self.product_class:
+        if self.product_class_id or self.product_class:
             return self.product_class
         if self.parent and self.parent.product_class:
             return self.parent.product_class
         return None
+    get_product_class.short_description = _("Product class")
+
+    # Images
 
     def get_missing_image(self):
         """
@@ -438,82 +557,61 @@ class AbstractProduct(models.Model):
 
     def primary_image(self):
         images = self.images.all()
-        if images.count():
+        try:
             return images[0]
-        # We return a dict with fields that mirror the key properties of the
-        # ProductImage class so this missing image can be used interchangably
-        # in templates.  Strategy pattern ftw!
-        return {
-            'original': self.get_missing_image(),
-            'caption': '',
-            'is_missing': True}
+        except IndexError:
+            # We return a dict with fields that mirror the key properties of
+            # the ProductImage class so this missing image can be used
+            # interchangably in templates.  Strategy pattern ftw!
+            return {
+                'original': self.get_missing_image(),
+                'caption': '',
+                'is_missing': True}
 
-    # Helpers
-
-    def _min_variant_price(self, property):
-        u"""Return minimum variant price"""
-        prices = []
-        for variant in self.variants.all():
-            if variant.has_stockrecord:
-                prices.append(getattr(variant.stockrecord, property))
-        if not prices:
-            return None
-        prices.sort()
-        return prices[0]
-
-    class Meta:
-        abstract = True
-        ordering = ['-date_created']
-        verbose_name = _('Product')
-        verbose_name_plural = _('Products')
-
-    def __unicode__(self):
-        if self.is_variant:
-            return u"%s (%s)" % (self.get_title(), self.attribute_summary())
-        return self.get_title()
-
-    @models.permalink
-    def get_absolute_url(self):
-        u"""Return a product's absolute url"""
-        return ('catalogue:detail', (), {
-            'product_slug': self.slug,
-            'pk': self.id})
-
-    def __init__(self, *args, **kwargs):
-        super(AbstractProduct, self).__init__(*args, **kwargs)
-        self.attr = ProductAttributesContainer(product=self)
-
-    def save(self, *args, **kwargs):
-        if self.is_top_level and not self.title:
-            raise ValidationError(_("Canonical products must have a title"))
-        if not self.slug:
-            self.slug = slugify(self.get_title())
-
-        # Validate attributes if necessary
-        self.attr.validate_attributes()
-
-        # Save product
-        super(AbstractProduct, self).save(*args, **kwargs)
-
-        # Finally, save attributes
-        self.attr.save()
+    # Updating methods
 
     def update_rating(self):
         """
-        Update rating field
+        Recalculate rating field
+        """
+        self.rating = self.calculate_rating()
+        self.save()
+    update_rating.alters_data = True
+
+    def calculate_rating(self):
+        """
+        Calculate rating value
         """
-        ProductReview = get_model('reviews', 'ProductReview')
         result = self.reviews.filter(
-            status=ProductReview.APPROVED
+            status=self.reviews.model.APPROVED
         ).aggregate(
             sum=Sum('score'), count=Count('id'))
         reviews_sum = result['sum'] or 0
         reviews_count = result['count'] or 0
+        rating = None
         if reviews_count > 0:
-            self.rating = float(reviews_sum) / reviews_count
+            rating = float(reviews_sum) / reviews_count
+        return rating
+
+    def has_review_by(self, user):
+        if user.is_anonymous():
+            return False
+        return self.reviews.filter(user=user).exists()
+
+    def is_review_permitted(self, user):
+        """
+        Determines whether a user may add a review on this product.
+
+        Default implementation respects OSCAR_ALLOW_ANON_REVIEWS and only
+        allows leaving one review per user and product.
+
+        Override this if you want to alter the default behaviour; e.g. enforce
+        that a user purchased the product to be allowed to leave a review.
+        """
+        if user.is_authenticated() or settings.OSCAR_ALLOW_ANON_REVIEWS:
+            return not self.has_review_by(user)
         else:
-            self.rating = None
-        self.save()
+            return False
 
 
 class ProductRecommendation(models.Model):
@@ -535,6 +633,10 @@ class ProductRecommendation(models.Model):
 class ProductAttributesContainer(object):
     """
     Stolen liberally from django-eav, but simplified to be product-specific
+
+    To set attributes on a product, use the `attr` attribute:
+
+        product.attr.weight = 125
     """
 
     def __setstate__(self, state):
@@ -554,7 +656,7 @@ class ProductAttributesContainer(object):
             return getattr(self, name)
         raise AttributeError(
             _("%(obj)s has no attribute named '%(attr)s'") % {
-                'obj': self.product.product_class, 'attr': name})
+                'obj': self.product.get_product_class(), 'attr': name})
 
     def validate_attributes(self):
         for attribute in self.get_all_attributes():
@@ -562,7 +664,7 @@ class ProductAttributesContainer(object):
             if value is None:
                 if attribute.required:
                     raise ValidationError(
-                        _("%(attr)s attribute cannot be blanke") %
+                        _("%(attr)s attribute cannot be blank") %
                         {'attr': attribute.code})
             else:
                 try:
@@ -599,6 +701,15 @@ class AbstractProductAttribute(models.Model):
     Defines an attribute for a product class. (For example, number_of_pages for
     a 'book' class)
     """
+    product_class = models.ForeignKey(
+        'catalogue.ProductClass', related_name='attributes', blank=True,
+        null=True, verbose_name=_("Product Class"))
+    name = models.CharField(_('Name'), max_length=128)
+    code = models.SlugField(
+        _('Code'), max_length=128,
+        validators=[RegexValidator(
+            regex=r'^[a-zA-Z_][0-9a-zA-Z_]*$',
+            message=_("Code must match ^[a-zA-Z_][0-9a-zA-Z_]*$"))])
 
     TYPE_CHOICES = (
         ("text", _("Text")),
@@ -608,15 +719,10 @@ class AbstractProductAttribute(models.Model):
         ("richtext", _("Rich Text")),
         ("date", _("Date")),
         ("option", _("Option")),
-        ("entity", _("Entity")))
-    product_class = models.ForeignKey(
-        'catalogue.ProductClass', related_name='attributes', blank=True,
-        null=True, verbose_name=_("Product Class"))
-    name = models.CharField(_('Name'), max_length=128)
-    code = models.SlugField(
-        _('Code'), max_length=128,
-        validators=[RegexValidator(regex=r'^[a-zA-Z_][0-9a-zA-Z_]*$',
-        message=_("Code must match ^[a-zA-Z_][0-9a-zA-Z_]*$"))])
+        ("entity", _("Entity")),
+        ("file", _("File")),
+        ("image", _("Image")),
+    )
     type = models.CharField(
         choices=TYPE_CHOICES, default=TYPE_CHOICES[0][0],
         max_length=20, verbose_name=_("Type"))
@@ -636,6 +742,14 @@ class AbstractProductAttribute(models.Model):
         verbose_name = _('Product Attribute')
         verbose_name_plural = _('Product Attributes')
 
+    @property
+    def is_option(self):
+        return self.type == "option"
+
+    @property
+    def is_file(self):
+        return self.type in ["file", "image"]
+
     def _validate_text(self, value):
         if not (type(value) == unicode or type(value) == str):
             raise ValidationError(_("Must be str or unicode"))
@@ -683,6 +797,10 @@ class AbstractProductAttribute(models.Model):
                 _("%(enum)s is not a valid choice for %(attr)s") %
                 {'enum': value, 'attr': self})
 
+    def _validate_file(self, value):
+        if value and not isinstance(value, File):
+            raise ValidationError(_("Must be a file field"))
+
     def get_validator(self):
         DATATYPE_VALIDATORS = {
             'text': self._validate_text,
@@ -693,6 +811,8 @@ class AbstractProductAttribute(models.Model):
             'date': self._validate_date,
             'entity': self._validate_entity,
             'option': self._validate_option,
+            'file': self._validate_file,
+            'image': self._validate_file,
         }
 
         return DATATYPE_VALIDATORS[self.type]
@@ -707,16 +827,34 @@ class AbstractProductAttribute(models.Model):
         try:
             value_obj = product.attribute_values.get(attribute=self)
         except get_model('catalogue', 'ProductAttributeValue').DoesNotExist:
-            if value is None or value == '':
+            # FileField uses False for anouncing deletion of the file
+            # not creating a new value
+            delete_file = self.is_file and value is False
+            if value is None or value == '' or delete_file:
                 return
             model = get_model('catalogue', 'ProductAttributeValue')
             value_obj = model.objects.create(product=product, attribute=self)
-        if value is None or value == '':
-            value_obj.delete()
-            return
-        if value != value_obj.value:
-            value_obj.value = value
-            value_obj.save()
+
+        if self.is_file:
+            # File fields in Django are treated differently, see
+            # django.db.models.fields.FileField and method save_form_data
+            if value is None:
+                # No change
+                return
+            elif value is False:
+                # Delete file
+                value_obj.delete()
+            else:
+                # New uploaded file
+                value_obj.value = value
+                value_obj.save()
+        else:
+            if value is None or value == '':
+                value_obj.delete()
+                return
+            if value != value_obj.value:
+                value_obj.value = value
+                value_obj.save()
 
     def validate_value(self, value):
         self.get_validator()(value)
@@ -748,7 +886,7 @@ class AbstractProductAttributeValue(models.Model):
     value_text = models.CharField(
         _('Text'), max_length=255, blank=True, null=True)
     value_integer = models.IntegerField(_('Integer'), blank=True, null=True)
-    value_boolean = models.BooleanField(_('Boolean'), blank=True)
+    value_boolean = models.NullBooleanField(_('Boolean'), blank=True)
     value_float = models.FloatField(_('Float'), blank=True, null=True)
     value_richtext = models.TextField(_('Richtext'), blank=True, null=True)
     value_date = models.DateField(_('Date'), blank=True, null=True)
@@ -758,6 +896,12 @@ class AbstractProductAttributeValue(models.Model):
     value_entity = models.ForeignKey(
         'catalogue.AttributeEntity', blank=True, null=True,
         verbose_name=_("Value Entity"))
+    value_file = models.FileField(
+        upload_to=settings.OSCAR_IMAGE_FOLDER, max_length=255,
+        blank=True, null=True)
+    value_image = models.ImageField(
+        upload_to=settings.OSCAR_IMAGE_FOLDER, max_length=255,
+        blank=True, null=True)
 
     def _get_value(self):
         return getattr(self, 'value_%s' % self.attribute.type)
@@ -774,7 +918,7 @@ class AbstractProductAttributeValue(models.Model):
     class Meta:
         abstract = True
         verbose_name = _('Product Attribute Value')
-        verbose_name_plural = _('Product Attribut Values')
+        verbose_name_plural = _('Product Attribute Values')
 
     def __unicode__(self):
         return u"%s: %s" % (self.attribute.name, self.value)
@@ -784,6 +928,7 @@ class AbstractAttributeOptionGroup(models.Model):
     """
     Defines a group of options that collectively may be used as an
     attribute type
+
     For example, Language
     """
     name = models.CharField(_('Name'), max_length=128)
@@ -796,6 +941,11 @@ class AbstractAttributeOptionGroup(models.Model):
         verbose_name = _('Attribute Option Group')
         verbose_name_plural = _('Attribute Option Groups')
 
+    @property
+    def option_summary(self):
+        options = [o.option for o in self.options.all()]
+        return ", ".join(options)
+
 
 class AbstractAttributeOption(models.Model):
     """
@@ -906,8 +1056,40 @@ class AbstractOption(models.Model):
 
 class MissingProductImage(object):
 
+    """
+    Mimics a Django file field by having a name property.
+
+    sorl-thumbnail requires all it's images to be in MEDIA_ROOT. This class
+    tries symlinking the default "missing image" image in STATIC_ROOT
+    into MEDIA_ROOT for convenience, as that is necessary every time an Oscar
+    project is setup. This avoids the less helpful NotFound IOError that would
+    be raised when sorl-thumbnail tries to access it.
+    """
+
     def __init__(self, name=None):
         self.name = name if name else settings.OSCAR_MISSING_IMAGE_URL
+        media_file_path = os.path.join(settings.MEDIA_ROOT, self.name)
+        # don't try to symlink if MEDIA_ROOT is not set (e.g. running tests)
+        if settings.MEDIA_ROOT and not os.path.exists(media_file_path):
+            self.symlink_missing_image(media_file_path)
+
+    def symlink_missing_image(self, media_file_path):
+        static_file_path = find('oscar/img/%s' % self.name)
+        if static_file_path is not None:
+            try:
+                os.symlink(static_file_path, media_file_path)
+            except OSError:
+                raise ImproperlyConfigured((
+                    "Please copy/symlink the "
+                    "'missing image' image at %s into your MEDIA_ROOT at %s. "
+                    "This exception was raised because Oscar was unable to "
+                    "symlink it for you.") % (media_file_path,
+                                              settings.MEDIA_ROOT))
+            else:
+                logging.info((
+                    "Symlinked the 'missing image' image at %s into your "
+                    "MEDIA_ROOT at %s") % (media_file_path,
+                                           settings.MEDIA_ROOT))
 
 
 class AbstractProductImage(models.Model):
@@ -917,14 +1099,15 @@ class AbstractProductImage(models.Model):
     product = models.ForeignKey(
         'catalogue.Product', related_name='images', verbose_name=_("Product"))
     original = models.ImageField(
-        _("Original"), upload_to=settings.OSCAR_IMAGE_FOLDER)
+        _("Original"), upload_to=settings.OSCAR_IMAGE_FOLDER, max_length=255)
     caption = models.CharField(
         _("Caption"), max_length=200, blank=True, null=True)
 
-    # Use display_order to determine which is the "primary" image
-    display_order = models.PositiveIntegerField(_("Display Order"), default=0,
-        help_text=_("""An image with a display order of
-                       zero will be the primary image for a product"""))
+    #: Use display_order to determine which is the "primary" image
+    display_order = models.PositiveIntegerField(
+        _("Display Order"), default=0,
+        help_text=_("An image with a display order of zero will be the primary"
+                    " image for a product"))
     date_created = models.DateTimeField(_("Date Created"), auto_now_add=True)
 
     class Meta:
@@ -942,19 +1125,3 @@ class AbstractProductImage(models.Model):
         Return bool if image display order is 0
         """
         return self.display_order == 0
-
-    def resized_image_url(self, width=None, height=None, **kwargs):
-        return self.original.url
-
-    @property
-    def fullsize_url(self):
-        """
-        Returns the URL path for this image.  This is intended
-        to be overridden in subclasses that want to serve
-        images in a specific way.
-        """
-        return self.resized_image_url()
-
-    @property
-    def thumbnail_url(self):
-        return self.resized_image_url()
