@@ -1,15 +1,87 @@
 import hashlib
 import random
 
+from django.conf import settings
+from django.contrib.auth import models as auth_models
+from django.core.urlresolvers import reverse
 from django.db import models
-from django.utils.translation import ugettext_lazy as _
 from django.template import Template, Context, TemplateDoesNotExist
 from django.template.loader import get_template
-from django.conf import settings
-from django.utils.timezone import now
-from django.core.urlresolvers import reverse
+from django.utils import timezone
+from django.utils.translation import ugettext_lazy as _
 
 from oscar.apps.customer.managers import CommunicationTypeManager
+from oscar.core.compat import AUTH_USER_MODEL
+
+if hasattr(auth_models, 'BaseUserManager'):
+    # Only define custom UserModel when Django >= 1.5
+    class UserManager(auth_models.BaseUserManager):
+
+        def create_user(self, email, password=None, **extra_fields):
+            """
+            Creates and saves a User with the given username, email and
+            password.
+            """
+            now = timezone.now()
+            if not email:
+                raise ValueError('The given email must be set')
+            email = UserManager.normalize_email(email)
+            user = self.model(
+                email=email, is_staff=False, is_active=True,
+                is_superuser=False,
+                last_login=now, date_joined=now, **extra_fields)
+
+            user.set_password(password)
+            user.save(using=self._db)
+            return user
+
+        def create_superuser(self, email, password, **extra_fields):
+            u = self.create_user(email, password, **extra_fields)
+            u.is_staff = True
+            u.is_active = True
+            u.is_superuser = True
+            u.save(using=self._db)
+            return u
+
+    class AbstractUser(auth_models.AbstractBaseUser,
+                       auth_models.PermissionsMixin):
+        """
+        An abstract base user suitable for use in Oscar projects.
+
+        This is basically a copy of the core AbstractUser model but without a
+        username field
+        """
+        email = models.EmailField(_('email address'), unique=True)
+        first_name = models.CharField(
+            _('First name'), max_length=255, blank=True)
+        last_name = models.CharField(
+            _('Last name'), max_length=255, blank=True)
+        is_staff = models.BooleanField(
+            _('Staff status'), default=False,
+            help_text=_('Designates whether the user can log into this admin '
+                        'site.'))
+        is_active = models.BooleanField(
+            _('Active'), default=True,
+            help_text=_('Designates whether this user should be treated as '
+                        'active. Unselect this instead of deleting accounts.'))
+        date_joined = models.DateTimeField(_('date joined'),
+                                           default=timezone.now)
+
+        objects = UserManager()
+
+        USERNAME_FIELD = 'email'
+
+        class Meta:
+            verbose_name = _('User')
+            verbose_name_plural = _('Users')
+            abstract = True
+
+        def get_full_name(self):
+            full_name = '%s %s' % (self.first_name, self.last_name)
+            return full_name.strip()
+
+        def get_short_name(self):
+            return self.first_name
 
 
 class AbstractEmail(models.Model):
@@ -17,7 +89,8 @@ class AbstractEmail(models.Model):
     This is a record of all emails sent to a customer.
     Normally, we only record order-related emails.
     """
-    user = models.ForeignKey('auth.User', related_name='emails', verbose_name=_("User"))
+    user = models.ForeignKey(AUTH_USER_MODEL, related_name='emails',
+                             verbose_name=_("User"))
     subject = models.TextField(_('Subject'), max_length=255)
     body_text = models.TextField(_("Body Text"))
     body_html = models.TextField(_("Body HTML"), blank=True, null=True)
@@ -42,7 +115,7 @@ class AbstractCommunicationEventType(models.Model):
     # eg. PASSWORD_RESET
     code = models.SlugField(_('Code'), max_length=128)
 
-    # Name is the friendly description of an event for use in the admin
+    #: Name is the friendly description of an event for use in the admin
     name = models.CharField(
         _('Name'), max_length=255,
         help_text=_("This is just used for organisational purposes"))
@@ -64,7 +137,8 @@ class AbstractCommunicationEventType(models.Model):
 
     # Template content for SMS messages
     sms_template = models.CharField(_('SMS Template'), max_length=170,
-                                    blank=True, help_text=_("SMS template"))
+                                    blank=True, null=True,
+                                    help_text=_("SMS template"))
 
     date_created = models.DateTimeField(_("Date Created"), auto_now_add=True)
     date_updated = models.DateTimeField(_("Date Updated"), auto_now=True)
@@ -109,12 +183,11 @@ class AbstractCommunicationEventType(models.Model):
                 except TemplateDoesNotExist:
                     templates[name] = None
 
-
         # Pass base URL for serving images within HTML emails
         if ctx is None:
             ctx = {}
-        ctx['static_base_url'] = getattr(settings,
-                                         'OSCAR_STATIC_BASE_URL', None)
+        ctx['static_base_url'] = getattr(
+            settings, 'OSCAR_STATIC_BASE_URL', None)
 
         messages = {}
         for name, template in templates.items():
@@ -122,6 +195,7 @@ class AbstractCommunicationEventType(models.Model):
 
         # Ensure the email subject doesn't contain any newlines
         messages['subject'] = messages['subject'].replace("\n", "")
+        messages['subject'] = messages['subject'].replace("\r", "")
 
         return messages
 
@@ -136,11 +210,11 @@ class AbstractCommunicationEventType(models.Model):
 
 
 class AbstractNotification(models.Model):
-    recipient = models.ForeignKey('auth.User', related_name='notifications',
-                                  db_index=True)
+    recipient = models.ForeignKey(AUTH_USER_MODEL,
+                                  related_name='notifications', db_index=True)
 
     # Not all notifications will have a sender.
-    sender = models.ForeignKey('auth.User', null=True)
+    sender = models.ForeignKey(AUTH_USER_MODEL, null=True)
 
     # HTML is allowed in this field as it can contain links
     subject = models.CharField(max_length=255)
@@ -158,7 +232,7 @@ class AbstractNotification(models.Model):
                                 default=INBOX)
 
     date_sent = models.DateTimeField(auto_now_add=True)
-    date_read = models.DateTimeField(null=True)
+    date_read = models.DateTimeField(blank=True, null=True)
 
     class Meta:
         ordering = ('-date_sent',)
@@ -186,8 +260,9 @@ class AbstractProductAlert(models.Model):
     # A user is only required if the notification is created by a
     # registered user, anonymous users will only have an email address
     # attached to the notification
-    user = models.ForeignKey('auth.User', db_index=True, blank=True, null=True,
-                             related_name="alerts", verbose_name=_('User'))
+    user = models.ForeignKey(AUTH_USER_MODEL, db_index=True, blank=True,
+                             null=True, related_name="alerts",
+                             verbose_name=_('User'))
     email = models.EmailField(_("Email"), db_index=True, blank=True, null=True)
 
     # This key are used to confirm and cancel alerts for anon users
@@ -242,19 +317,19 @@ class AbstractProductAlert(models.Model):
 
     def confirm(self):
         self.status = self.ACTIVE
-        self.date_confirmed = now()
+        self.date_confirmed = timezone.now()
         self.save()
     confirm.alters_data = True
 
     def cancel(self):
         self.status = self.CANCELLED
-        self.date_cancelled = now()
+        self.date_cancelled = timezone.now()
         self.save()
     cancel.alters_data = True
 
     def close(self):
         self.status = self.CLOSED
-        self.date_closed = now()
+        self.date_closed = timezone.now()
         self.save()
     close.alters_data = True
 
@@ -271,11 +346,12 @@ class AbstractProductAlert(models.Model):
         # Ensure date fields get updated when saving from modelform (which just
         # calls save, and doesn't call the methods cancel(), confirm() etc).
         if self.status == self.CANCELLED and self.date_cancelled is None:
-            self.date_cancelled = now()
-        if not self.user and self.status == self.ACTIVE and self.date_confirmed is None:
-            self.date_confirmed = now()
+            self.date_cancelled = timezone.now()
+        if not self.user and self.status == self.ACTIVE \
+                and self.date_confirmed is None:
+            self.date_confirmed = timezone.now()
         if self.status == self.CLOSED and self.date_closed is None:
-            self.date_closed = now()
+            self.date_closed = timezone.now()
 
         return super(AbstractProductAlert, self).save(*args, **kwargs)
 
@@ -290,4 +366,5 @@ class AbstractProductAlert(models.Model):
         return reverse('customer:alerts-confirm', kwargs={'key': self.key})
 
     def get_cancel_url(self):
-        return reverse('customer:alerts-cancel', kwargs={'key': self.key})
+        return reverse('customer:alerts-cancel-by-key', kwargs={'key':
+                                                                self.key})
