@@ -1,12 +1,21 @@
 from django.conf.urls import patterns, url
 
-from oscar.views.decorators import staff_member_required
 from oscar.core.application import Application
 from oscar.apps.dashboard.orders import views
 
 
 class OrdersDashboardApplication(Application):
     name = None
+    default_permissions = ['is_staff', ]
+    permissions_map = {
+        'order-list': (['is_staff'], ['partner.dashboard_access']),
+        'order-stats': (['is_staff'], ['partner.dashboard_access']),
+        'order-detail': (['is_staff'], ['partner.dashboard_access']),
+        'order-detail-note': (['is_staff'], ['partner.dashboard_access']),
+        'order-line-detail': (['is_staff'], ['partner.dashboard_access']),
+        'order-shipping-address': (['is_staff'], ['partner.dashboard_access']),
+    }
+
     order_list_view = views.OrderListView
     order_detail_view = views.OrderDetailView
     shipping_address_view = views.ShippingAddressUpdateView
@@ -14,9 +23,10 @@ class OrdersDashboardApplication(Application):
     order_stats_view = views.OrderStatsView
 
     def get_urls(self):
-        urlpatterns = patterns('',
+        urls = [
             url(r'^$', self.order_list_view.as_view(), name='order-list'),
-            url(r'^statistics/$', self.order_stats_view.as_view(), name='order-stats'),
+            url(r'^statistics/$', self.order_stats_view.as_view(),
+                name='order-stats'),
             url(r'^(?P<number>[-\w]+)/$',
                 self.order_detail_view.as_view(), name='order-detail'),
             url(r'^(?P<number>[-\w]+)/notes/(?P<note_id>\d+)/$',
@@ -24,12 +34,10 @@ class OrdersDashboardApplication(Application):
             url(r'^(?P<number>[-\w]+)/lines/(?P<line_id>\d+)/$',
                 self.line_detail_view.as_view(), name='order-line-detail'),
             url(r'^(?P<number>[-\w]+)/shipping-address/$',
-                self.shipping_address_view.as_view(), name='order-shipping-address'),
-        )
-        return self.post_process_urls(urlpatterns)
-
-    def get_url_decorator(self, url_name):
-        return staff_member_required
+                self.shipping_address_view.as_view(),
+                name='order-shipping-address'),
+        ]
+        return self.post_process_urls(patterns('', *urls))
 
 
 application = OrdersDashboardApplication()
