@@ -9,13 +9,14 @@ from django.db.models.loading import get_model
 from django.db.models import fields, Q, Sum, Count
 from django.http import HttpResponse, HttpResponseRedirect, Http404
 from django.shortcuts import get_object_or_404
-from django.template.defaultfilters import date as format_date
 from django.utils.datastructures import SortedDict
 from django.views.generic import ListView, DetailView, UpdateView, FormView
 from django.conf import settings
 
 from oscar.core.loading import get_class
+from oscar.core.utils import format_datetime
 from oscar.apps.dashboard.orders import forms
+from oscar.views import sort_queryset
 from oscar.views.generic import BulkEditMixin
 from oscar.apps.dashboard.reports.csv_utils import CsvUnicodeWriter
 from oscar.apps.payment.exceptions import PaymentError
@@ -24,13 +25,41 @@ from oscar.apps.order.exceptions import InvalidShippingEvent, InvalidStatus
 Order = get_model('order', 'Order')
 OrderNote = get_model('order', 'OrderNote')
 ShippingAddress = get_model('order', 'ShippingAddress')
+Transaction = get_model('payment', 'Transaction')
 Line = get_model('order', 'Line')
 ShippingEventType = get_model('order', 'ShippingEventType')
 PaymentEventType = get_model('order', 'PaymentEventType')
 EventHandler = get_class('order.processing', 'EventHandler')
+Partner = get_model('partner', 'Partner')
+
+
+def queryset_orders_for_user(user):
+    """
+    Returns a queryset of all orders that a user is allowed to access.
+    A staff user may access all orders.
+    To allow access to an order for a non-staff user, at least one line's
+    partner has to have the user in the partner's list.
+    """
+    queryset = Order._default_manager.all()
+    if user.is_staff:
+        return queryset
+    else:
+        partners = Partner._default_manager.filter(users=user)
+        return queryset.filter(lines__partner__in=partners).distinct()
+
+
+def get_order_for_user_or_404(user, number):
+    try:
+        return queryset_orders_for_user(user).get(number=number)
+    except ObjectDoesNotExist:
+        raise Http404()
 
 
 class OrderStatsView(FormView):
+    """
+    Dashboard view for order statistics.
+    Supports the permission-based dashboard.
+    """
     template_name = 'dashboard/orders/statistics.html'
     form_class = forms.OrderStatsForm
 
@@ -55,7 +84,7 @@ class OrderStatsView(FormView):
         return ctx
 
     def get_stats(self, filters):
-        orders = Order.objects.filter(**filters)
+        orders = queryset_orders_for_user(self.request.user).filter(**filters)
         stats = {
             'total_orders': orders.count(),
             'total_lines': Line.objects.filter(order__in=orders).count(),
@@ -67,36 +96,47 @@ class OrderStatsView(FormView):
         return stats
 
 
-class OrderListView(ListView, BulkEditMixin):
+class OrderListView(BulkEditMixin, ListView):
+    """
+    Dashboard view for a list of orders.
+    Supports the permission-based dashboard.
+    """
     model = Order
     context_object_name = 'orders'
     template_name = 'dashboard/orders/order_list.html'
     form_class = forms.OrderSearchForm
     desc_template = _("%(main_filter)s %(name_filter)s %(title_filter)s"
                       "%(upc_filter)s %(sku_filter)s %(date_filter)s"
-                      "%(voucher_filter)s %(payment_filter)s %(status_filter)s")
+                      "%(voucher_filter)s %(payment_filter)s"
+                      "%(status_filter)s")
     paginate_by = 25
     description = ''
     actions = ('download_selected_orders',)
     current_view = 'dashboard:order-list'
 
+    def dispatch(self, request, *args, **kwargs):
+        # base_queryset is equal to all orders the user is allowed to access
+        self.base_queryset = queryset_orders_for_user(
+            request.user).order_by('-date_placed')
+        return super(OrderListView, self).dispatch(request, *args, **kwargs)
+
     def get(self, request, *args, **kwargs):
-        if 'order_number' in request.GET and request.GET.get('response_format', 'html') == 'html':
+        if 'order_number' in request.GET and request.GET.get(
+                'response_format', 'html') == 'html':
+            # Redirect to Order detail page if valid order number is given
             try:
-                order = Order.objects.get(number=request.GET['order_number'])
+                order = self.base_queryset.get(
+                    number=request.GET['order_number'])
             except Order.DoesNotExist:
                 pass
             else:
-                return HttpResponseRedirect(reverse('dashboard:order-detail', kwargs={'number': order.number}))
+                url = reverse('dashboard:order-detail',
+                              kwargs={'number': order.number})
+                return HttpResponseRedirect(url)
         return super(OrderListView, self).get(request, *args, **kwargs)
 
-    def get_queryset(self):
-        """
-        Build the queryset for this list and also update the title that
-        describes the queryset
-        """
-        queryset = self.model.objects.all().order_by('-date_placed')
-        queryset = self.sort_queryset(queryset)
+    def get_desc_context(self, data=None):  # noqa (too complex (16))
+        """Update the title that describes the queryset"""
         desc_ctx = {
             'main_filter': _('All orders'),
             'name_filter': '',
@@ -109,20 +149,79 @@ class OrderListView(ListView, BulkEditMixin):
             'status_filter': '',
         }
 
+        if 'order_status' in self.request.GET:
+            status = self.request.GET['order_status']
+            if status.lower() == 'none':
+                desc_ctx['main_filter'] = _("Orders without an order status")
+            else:
+                desc_ctx['main_filter'] = _("Orders with status '%s'") % status
+        if data is None:
+            return desc_ctx
+
+        if data['order_number']:
+            desc_ctx['main_filter'] = _('Orders with number starting with'
+                                        ' "%(order_number)s"') % data
+
+        if data['name']:
+            desc_ctx['name_filter'] = _(" with customer name matching"
+                                        " '%(name)s'") % data
+
+        if data['product_title']:
+            desc_ctx['title_filter'] \
+                = _(" including an item with title matching"
+                    " '%(product_title)s'") % data
+
+        if data['upc']:
+            desc_ctx['upc_filter'] = _(" including an item with UPC"
+                                       " '%(upc)s'") % data
+
+        if data['partner_sku']:
+            desc_ctx['upc_filter'] = _(" including an item with ID"
+                                       " '%(partner_sku)s'") % data
+
+        if data['date_from'] and data['date_to']:
+            desc_ctx['date_filter'] \
+                = _(" placed between %(start_date)s and %(end_date)s") \
+                % {'start_date': format_datetime(data['date_from']),
+                   'end_date': format_datetime(data['date_to'])}
+        elif data['date_from']:
+            desc_ctx['date_filter'] = _(" placed since %s") \
+                % format_datetime(data['date_from'])
+        elif data['date_to']:
+            date_to = data['date_to'] + datetime.timedelta(days=1)
+            desc_ctx['date_filter'] = _(" placed before %s") \
+                % format_datetime(date_to)
+        if data['voucher']:
+            desc_ctx['voucher_filter'] = _(" using voucher '%(voucher)s'") \
+                % data
+
+        if data['payment_method']:
+            desc_ctx['payment_filter'] = _(" paid for by %(payment_method)s") \
+                % data
+
+        if data['status']:
+            desc_ctx['status_filter'] = _(" with status %(status)s") % data
+
+        return desc_ctx
+
+    def get_queryset(self):  # noqa (too complex (19))
+        """
+        Build the queryset for this list.
+        """
+        queryset = sort_queryset(self.base_queryset, self.request,
+                                 ['number', 'total_incl_tax'])
+
         # Look for shortcut query filters
         if 'order_status' in self.request.GET:
             self.form = self.form_class()
             status = self.request.GET['order_status']
             if status.lower() == 'none':
-                desc_ctx['main_filter'] = _("Orders without an order status")
                 status = None
-            else:
-                desc_ctx['main_filter'] = _("Orders with status '%s'") % status
-            self.description = self.desc_template % desc_ctx
-            return self.model.objects.filter(status=status)
+            self.description = self.desc_template % self.get_desc_context()
+            return self.base_queryset.filter(status=status)
 
         if 'order_number' not in self.request.GET:
-            self.description = self.desc_template % desc_ctx
+            self.description = self.desc_template % self.get_desc_context()
             self.form = self.form_class()
             return queryset
 
@@ -133,11 +232,12 @@ class OrderListView(ListView, BulkEditMixin):
         data = self.form.cleaned_data
 
         if data['order_number']:
-            queryset = self.model.objects.filter(number__istartswith=data['order_number'])
-            desc_ctx['main_filter'] = _('Orders with number starting with "%s"') % data['order_number']
+            queryset = self.base_queryset.objects.filter(
+                number__istartswith=data['order_number'])
 
         if data['name']:
-            # If the value is two words, then assume they are first name and last name
+            # If the value is two words, then assume they are first name and
+            # last name
             parts = data['name'].split()
             allow_anon = getattr(settings, 'OSCAR_ALLOW_ANON_CHECKOUT', False)
 
@@ -146,66 +246,49 @@ class OrderListView(ListView, BulkEditMixin):
             else:
                 parts = [parts[0], parts[1:]]
 
-            filter = Q(user__first_name__istartswith=parts[0]) |\
-                     Q(user__last_name__istartswith=parts[1])
+            filter = Q(user__first_name__istartswith=parts[0])
+            filter |= Q(user__last_name__istartswith=parts[1])
             if allow_anon:
-                filter |= Q(billing_address__first_name__istartswith=parts[0]) |\
-                          Q(shipping_address__first_name__istartswith=parts[0]) |\
-                          Q(billing_address__last_name__istartswith=parts[1]) |\
-                          Q(shipping_address__last_name__istartswith=parts[1])
+                filter |= Q(billing_address__first_name__istartswith=parts[0])
+                filter |= Q(shipping_address__first_name__istartswith=parts[0])
+                filter |= Q(billing_address__last_name__istartswith=parts[1])
+                filter |= Q(shipping_address__last_name__istartswith=parts[1])
 
             queryset = queryset.filter(filter).distinct()
-            desc_ctx['name_filter'] = _(" with customer name matching '%s'") % data['name']
 
         if data['product_title']:
-            queryset = queryset.filter(lines__title__istartswith=data['product_title']).distinct()
-            desc_ctx['title_filter'] = _(" including an item with title matching '%s'") % data['product_title']
+            queryset = queryset.filter(
+                lines__title__istartswith=data['product_title']).distinct()
 
         if data['upc']:
             queryset = queryset.filter(lines__upc=data['upc'])
-            desc_ctx['upc_filter'] = _(" including an item with UPC '%s'") % data['upc']
 
         if data['partner_sku']:
             queryset = queryset.filter(lines__partner_sku=data['partner_sku'])
-            desc_ctx['upc_filter'] = _(" including an item with ID '%s'") % data['partner_sku']
 
         if data['date_from'] and data['date_to']:
             # Add 24 hours to make search inclusive
             date_to = data['date_to'] + datetime.timedelta(days=1)
-            queryset = queryset.filter(date_placed__gte=data['date_from']).filter(date_placed__lt=date_to)
-            desc_ctx['date_filter'] = _(" placed between %(start_date)s and %(end_date)s") % {
-                'start_date': format_date(data['date_from']),
-                'end_date': format_date(data['date_to'])}
+            queryset = queryset.filter(date_placed__gte=data['date_from'])
+            queryset = queryset.filter(date_placed__lt=date_to)
         elif data['date_from']:
             queryset = queryset.filter(date_placed__gte=data['date_from'])
-            desc_ctx['date_filter'] = _(" placed since %s") % format_date(data['date_from'])
         elif data['date_to']:
             date_to = data['date_to'] + datetime.timedelta(days=1)
             queryset = queryset.filter(date_placed__lt=date_to)
-            desc_ctx['date_filter'] = _(" placed before %s") % format_date(data['date_to'])
 
         if data['voucher']:
-            queryset = queryset.filter(discounts__voucher_code=data['voucher']).distinct()
-            desc_ctx['voucher_filter'] = _(" using voucher '%s'") % data['voucher']
+            queryset = queryset.filter(
+                discounts__voucher_code=data['voucher']).distinct()
 
         if data['payment_method']:
-            queryset = queryset.filter(sources__source_type__code=data['payment_method']).distinct()
-            desc_ctx['payment_filter'] = _(" paid for by %s") % data['payment_method']
+            queryset = queryset.filter(
+                sources__source_type__code=data['payment_method']).distinct()
 
         if data['status']:
             queryset = queryset.filter(status=data['status'])
-            desc_ctx['status_filter'] = _(" with status %s") % data['status']
 
-        self.description = self.desc_template % desc_ctx
-        return queryset
-
-    def sort_queryset(self, queryset):
-        sort = self.request.GET.get('sort', None)
-        allowed_sorts = ['number',]
-        if sort in allowed_sorts:
-            direction = self.request.GET.get('dir', 'desc')
-            sort = ('-' if direction == 'desc' else '') + sort
-            queryset = queryset.order_by(sort)
+        self.description = self.desc_template % self.get_desc_context(data)
         return queryset
 
     def get_context_data(self, **kwargs):
@@ -232,8 +315,9 @@ class OrderListView(ListView, BulkEditMixin):
         return 'orders.csv'
 
     def download_selected_orders(self, request, orders):
-        response = HttpResponse(mimetype='text/csv')
-        response['Content-Disposition'] = 'attachment; filename=%s' % self.get_download_filename(request)
+        response = HttpResponse(content_type='text/csv')
+        response['Content-Disposition'] = 'attachment; filename=%s' \
+            % self.get_download_filename(request)
         writer = CsvUnicodeWriter(response, delimiter=',')
 
         meta_data = (('number', _('Order number')),
@@ -254,25 +338,30 @@ class OrderListView(ListView, BulkEditMixin):
             row = columns.copy()
             row['number'] = order.number
             row['value'] = order.total_incl_tax
-            row['date'] = format_date(order.date_placed, 'DATETIME_FORMAT')
+            row['date'] = format_datetime(order.date_placed, 'DATETIME_FORMAT')
             row['num_items'] = order.num_items
             row['status'] = order.status
             row['customer'] = order.email
             if order.shipping_address:
-                row['shipping_address_name'] = order.shipping_address.name()
+                row['shipping_address_name'] = order.shipping_address.name
             else:
                 row['shipping_address_name'] = ''
             if order.billing_address:
-                row['billing_address_name'] = order.billing_address.name()
+                row['billing_address_name'] = order.billing_address.name
             else:
                 row['billing_address_name'] = ''
 
-            encoded_values = [unicode(value).encode('utf8') for value in row.values()]
+            encoded_values = [unicode(value).encode('utf8')
+                              for value in row.values()]
             writer.writerow(encoded_values)
         return response
 
 
 class OrderDetailView(DetailView):
+    """
+    Dashboard view to display a single order.
+    Supports the permission-based dashboard.
+    """
     model = Order
     context_object_name = 'order'
     template_name = 'dashboard/orders/order_detail.html'
@@ -282,7 +371,8 @@ class OrderDetailView(DetailView):
                     'create_payment_event')
 
     def get_object(self, queryset=None):
-        return get_object_or_404(self.model, number=self.kwargs['number'])
+        return get_order_for_user_or_404(self.request.user,
+                                         self.kwargs['number'])
 
     def get_context_data(self, **kwargs):
         ctx = super(OrderDetailView, self).get_context_data(**kwargs)
@@ -291,8 +381,13 @@ class OrderDetailView(DetailView):
         ctx['line_statuses'] = Line.all_statuses()
         ctx['shipping_event_types'] = ShippingEventType.objects.all()
         ctx['payment_event_types'] = PaymentEventType.objects.all()
+        ctx['payment_transactions'] = self.get_payment_transactions()
         return ctx
 
+    def get_payment_transactions(self):
+        return Transaction.objects.filter(
+            source__order=self.object)
+
     def get_order_note_form(self):
         post_data = None
         kwargs = {}
@@ -331,16 +426,19 @@ class OrderDetailView(DetailView):
                     qty = request.POST.get('selected_line_qty_%s' % line_id)
                     line_quantities.append(int(qty))
                 lines = order.lines.filter(id__in=line_ids)
-                if lines.count() == 0:
-                    messages.error(self.request, _("You must select some lines to act on"))
+                if not lines.exists():
+                    messages.error(self.request,
+                                   _("You must select some lines to act on"))
                     return self.reload_page_response()
-                return getattr(self, line_action)(request, order, lines, line_quantities)
+                return getattr(self, line_action)(request, order, lines,
+                                                  line_quantities)
 
         messages.error(request, _("No valid action submitted"))
         return self.reload_page_response()
 
     def reload_page_response(self, fragment=None):
-        url = reverse('dashboard:order-detail', kwargs={'number': self.object.number})
+        url = reverse('dashboard:order-detail', kwargs={'number':
+                                                        self.object.number})
         if fragment:
             url += '#' + fragment
         return HttpResponseRedirect(url)
@@ -371,47 +469,51 @@ class OrderDetailView(DetailView):
     def change_order_status(self, request, order):
         new_status = request.POST['new_status'].strip()
         if not new_status:
-            messages.error(request, _("The new status '%s' is not valid") % new_status)
+            messages.error(request, _("The new status '%s' is not valid")
+                           % new_status)
             return self.reload_page_response()
         if not new_status in order.available_statuses():
-            messages.error(request, _("The new status '%s' is not valid for this order") % new_status)
+            messages.error(request, _("The new status '%s' is not valid for"
+                                      " this order") % new_status)
             return self.reload_page_response()
 
-        handler = EventHandler()
+        handler = EventHandler(request.user)
         try:
             handler.handle_order_status_change(order, new_status)
         except PaymentError, e:
-            messages.error(request, _("Unable to change order status due to payment error: %s") % e)
+            messages.error(request, _("Unable to change order status due to"
+                                      " payment error: %s") % e)
         else:
-            msg = _("Order status changed from '%(old_status)s' to '%(new_status)s'") % {
-                'old_status': order.status,
-                'new_status': new_status}
+            msg = _("Order status changed from '%(old_status)s' to"
+                    " '%(new_status)s'") % {'old_status': order.status,
+                                            'new_status': new_status}
             messages.info(request, msg)
             order.notes.create(user=request.user, message=msg,
-                            note_type=OrderNote.SYSTEM)
+                               note_type=OrderNote.SYSTEM)
         return self.reload_page_response(fragment='activity')
 
     def change_line_statuses(self, request, order, lines, quantities):
         new_status = request.POST['new_status'].strip()
         if not new_status:
-            messages.error(request, _("The new status '%s' is not valid") % new_status)
+            messages.error(request, _("The new status '%s' is not valid")
+                           % new_status)
             return self.reload_page_response()
         errors = []
         for line in lines:
             if new_status not in line.available_statuses():
-                errors.append(_("'%(status)s' is not a valid new status for line %(line_id)d") % {
-                    'status': new_status,
-                    'line_id': line.id})
+                errors.append(_("'%(status)s' is not a valid new status for"
+                                " line %(line_id)d") % {'status': new_status,
+                                                        'line_id': line.id})
         if errors:
             messages.error(request, "\n".join(errors))
             return self.reload_page_response()
 
         msgs = []
         for line in lines:
-            msg = _("Status of line #%(line_id)d changed from '%(old_status)s' to '%(new_status)s'") % {
-                        'line_id': line.id,
-                        'old_status': line.status,
-                        'new_status': new_status}
+            msg = _("Status of line #%(line_id)d changed from '%(old_status)s'"
+                    " to '%(new_status)s'") % {'line_id': line.id,
+                                               'old_status': line.status,
+                                               'new_status': new_status}
             msgs.append(msg)
             line.set_status(new_status)
         message = "\n".join(msgs)
@@ -425,7 +527,8 @@ class OrderDetailView(DetailView):
         try:
             event_type = ShippingEventType._default_manager.get(code=code)
         except ShippingEventType.DoesNotExist:
-            messages.error(request, _("The event type '%s' is not valid") % code)
+            messages.error(request, _("The event type '%s' is not valid")
+                           % code)
             return self.reload_page_response()
 
         reference = request.POST.get('reference', None)
@@ -434,11 +537,14 @@ class OrderDetailView(DetailView):
                                                  quantities,
                                                  reference=reference)
         except InvalidShippingEvent, e:
-            messages.error(request, _("Unable to create shipping event: %s") % e)
+            messages.error(request,
+                           _("Unable to create shipping event: %s") % e)
         except InvalidStatus, e:
-            messages.error(request, _("Unable to create shipping event: %s") % e)
+            messages.error(request,
+                           _("Unable to create shipping event: %s") % e)
         except PaymentError, e:
-            messages.error(request, _("Unable to create shipping event due to payment error: %s") % e)
+            messages.error(request, _("Unable to create shipping event due to"
+                                      " payment error: %s") % e)
         else:
             messages.success(request, _("Shipping event created"))
         return self.reload_page_response()
@@ -458,13 +564,15 @@ class OrderDetailView(DetailView):
         try:
             event_type = PaymentEventType._default_manager.get(code=code)
         except PaymentEventType.DoesNotExist:
-            messages.error(request, _("The event type '%s' is not valid") % code)
+            messages.error(request, _("The event type '%s' is not valid")
+                           % code)
             return self.reload_page_response()
         try:
             EventHandler().handle_payment_event(order, event_type, amount,
                                                 lines, quantities)
         except PaymentError, e:
-            messages.error(request, _("Unable to change order status due to payment error: %s") % e)
+            messages.error(request, _("Unable to change order status due to"
+                                      " payment error: %s") % e)
         else:
             messages.info(request, _("Payment event created"))
         return self.reload_page_response()
@@ -489,13 +597,19 @@ class OrderDetailView(DetailView):
 
 
 class LineDetailView(DetailView):
+    """
+    Dashboard view to show a single line of an order.
+    Supports the permission-based dashboard.
+    """
     model = Line
     context_object_name = 'line'
     template_name = 'dashboard/orders/line_detail.html'
 
     def get_object(self, queryset=None):
+        order = get_order_for_user_or_404(self.request.user,
+                                          self.kwargs['number'])
         try:
-            return self.model.objects.get(pk=self.kwargs['line_id'])
+            return order.lines.get(pk=self.kwargs['line_id'])
         except self.model.DoesNotExist:
             raise Http404()
 
@@ -513,11 +627,14 @@ def get_changes_between_models(model1, model2, excludes=None):
         excludes = []
     changes = {}
     for field in model1._meta.fields:
-        if not (isinstance(field, (fields.AutoField, fields.related.RelatedField))
+        if (isinstance(field, (fields.AutoField,
+                               fields.related.RelatedField))
                 or field.name in excludes):
-            if field.value_from_object(model1) != field.value_from_object(model2):
-                changes[field.verbose_name] = (field.value_from_object(model1),
-                                               field.value_from_object(model2))
+            continue
+
+        if field.value_from_object(model1) != field.value_from_object(model2):
+            changes[field.verbose_name] = (field.value_from_object(model1),
+                                           field.value_from_object(model2))
     return changes
 
 
@@ -528,21 +645,28 @@ def get_change_summary(model1, model2):
     changes = get_changes_between_models(model1, model2, ['search_text'])
     change_descriptions = []
     for field, delta in changes.items():
-        change_descriptions.append(_("%(field)s changed from '%(old_value)s' to '%(new_value)s'") % {
-            'field': field,
-            'old_value': delta[0],
-            'new_value': delta[1]})
+        change_descriptions.append(_("%(field)s changed from '%(old_value)s'"
+                                     " to '%(new_value)s'")
+                                   % {'field': field,
+                                      'old_value': delta[0],
+                                      'new_value': delta[1]})
     return "\n".join(change_descriptions)
 
 
 class ShippingAddressUpdateView(UpdateView):
+    """
+    Dashboard view to update an order's shipping address.
+    Supports the permission-based dashboard.
+    """
     model = ShippingAddress
     context_object_name = 'address'
     template_name = 'dashboard/orders/shippingaddress_form.html'
     form_class = forms.ShippingAddressForm
 
     def get_object(self, queryset=None):
-        return get_object_or_404(self.model, order__number=self.kwargs['number'])
+        order = get_order_for_user_or_404(self.request.user,
+                                          self.kwargs['number'])
+        return get_object_or_404(self.model, order=order)
 
     def get_context_data(self, **kwargs):
         ctx = super(ShippingAddressUpdateView, self).get_context_data(**kwargs)
@@ -556,9 +680,10 @@ class ShippingAddressUpdateView(UpdateView):
         if changes:
             msg = _("Delivery address updated:\n%s") % changes
             self.object.order.notes.create(user=self.request.user, message=msg,
-                                        note_type=OrderNote.SYSTEM)
+                                           note_type=OrderNote.SYSTEM)
         return response
 
     def get_success_url(self):
         messages.info(self.request, _("Delivery address updated"))
-        return reverse('dashboard:order-detail', kwargs={'number': self.object.order.number, })
+        return reverse('dashboard:order-detail',
+                       kwargs={'number': self.object.order.number, })
