@@ -6,9 +6,10 @@ StockRecord = get_model('partner', 'StockRecord')
 
 
 class StockRecordAdmin(admin.ModelAdmin):
-    list_display = ('product', 'partner', 'partner_sku', 'price_excl_tax', 'cost_price', 'num_in_stock')
+    list_display = ('product', 'partner', 'partner_sku', 'price_excl_tax',
+                    'cost_price', 'num_in_stock')
     list_filter = ('partner',)
-    
+
 
 admin.site.register(Partner)
 admin.site.register(StockRecord, StockRecordAdmin)
