@@ -2,7 +2,8 @@ from django.dispatch import receiver
 from django.db.models.signals import post_save
 
 from oscar.core.loading import get_classes
-StockRecord, StockAlert = get_classes('partner.models', ('StockRecord', 'StockAlert'))
+StockRecord, StockAlert = get_classes('partner.models', ['StockRecord',
+                                                         'StockAlert'])
 
 
 @receiver(post_save, sender=StockRecord)
