@@ -1,16 +1,13 @@
 from django.db import models
 from django.conf import settings
-from django.utils.translation import ugettext as _
+from django.utils.translation import ugettext_lazy as _
 from django.core.urlresolvers import reverse
 from django.contrib.contenttypes.models import ContentType
 from django.contrib.contenttypes import generic
 from django.db.models import get_model
-from django.utils.translation import ugettext_lazy as _
 
 from oscar.models.fields import ExtendedURLField
 
-Item = get_model('product', 'Item')
-
 
 # Linking models - these link promotions to content (eg pages, or keywords)
 
@@ -22,7 +19,8 @@ class LinkedPromotion(models.Model):
     object_id = models.PositiveIntegerField()
     content_object = generic.GenericForeignKey('content_type', 'object_id')
 
-    position = models.CharField(_("Position"), max_length=100, help_text="Position on page")
+    position = models.CharField(_("Position"), max_length=100,
+                                help_text="Position on page")
     display_order = models.PositiveIntegerField(_("Display Order"), default=0)
     clicks = models.PositiveIntegerField(_("Clicks"), default=0)
     date_created = models.DateTimeField(_("Date Created"), auto_now_add=True)
@@ -43,13 +41,15 @@ class PagePromotion(LinkedPromotion):
     """
     A promotion embedded on a particular page.
     """
-    page_url = ExtendedURLField(_('Page URL'), max_length=128, db_index=True)
+    page_url = ExtendedURLField(
+        _('Page URL'), max_length=128, db_index=True, verify_exists=True)
 
     def __unicode__(self):
         return u"%s on %s" % (self.content_object, self.page_url)
 
     def get_link(self):
-        return reverse('promotions:page-click', kwargs={'page_promotion_id': self.id})
+        return reverse('promotions:page-click',
+                       kwargs={'page_promotion_id': self.id})
 
     class Meta:
         verbose_name = _("Page Promotion")
@@ -68,10 +68,11 @@ class KeywordPromotion(LinkedPromotion):
 
     # We allow an additional filter which will let search query matches
     # be restricted to different parts of the site.
-    filter = models.CharField(_("Filter"), max_length=200, blank=True, null=True)
+    filter = models.CharField(_("Filter"), max_length=200, blank=True)
 
     def get_link(self):
-        return reverse('promotions:keyword-click', kwargs={'keyword_promotion_id': self.id})
+        return reverse('promotions:keyword-click',
+                       kwargs={'keyword_promotion_id': self.id})
 
     class Meta:
         verbose_name = _("Keyword Promotion")
@@ -86,7 +87,8 @@ class AbstractPromotion(models.Model):
     that subclasses must implement.
     """
     _type = 'Promotion'
-    keywords = generic.GenericRelation(KeywordPromotion, verbose_name=_('Keywords'))
+    keywords = generic.GenericRelation(KeywordPromotion,
+                                       verbose_name=_('Keywords'))
     pages = generic.GenericRelation(PagePromotion, verbose_name=_('Pages'))
 
     class Meta:
@@ -124,8 +126,9 @@ class AbstractPromotion(models.Model):
         ctype = self.content_type
         page_count = PagePromotion.objects.filter(content_type=ctype,
                                                   object_id=self.id).count()
-        keyword_count = KeywordPromotion.objects.filter(content_type=ctype,
-                                                        object_id=self.id).count()
+        keyword_count \
+            = KeywordPromotion.objects.filter(content_type=ctype,
+                                              object_id=self.id).count()
         return page_count + keyword_count
 
 
@@ -140,9 +143,9 @@ class RawHTML(AbstractPromotion):
     # if a different width container is required).  This isn't always
     # required.
     display_type = models.CharField(
-        _("Display type"), max_length=128,
-        blank=True, null=True,
-        help_text=_("This can be used to have different types of HTML blocks (eg different widths)"))
+        _("Display type"), max_length=128, blank=True,
+        help_text=_("This can be used to have different types of HTML blocks"
+                    " (eg different widths)"))
     body = models.TextField(_("HTML"))
     date_created = models.DateTimeField(auto_now_add=True)
 
@@ -163,9 +166,12 @@ class Image(AbstractPromotion):
     """
     _type = 'Image'
     name = models.CharField(_("Name"), max_length=128)
-    link_url = ExtendedURLField(_('Link URL'), blank=True, null=True, help_text=_("""This is
-        where this promotion links to"""))
-    image = models.ImageField(_('Image'), upload_to=settings.OSCAR_PROMOTION_FOLDER)
+    link_url = ExtendedURLField(
+        _('Link URL'), blank=True,
+        help_text=_('This is where this promotion links to'))
+    image = models.ImageField(
+        _('Image'), upload_to=settings.OSCAR_PROMOTION_FOLDER,
+        max_length=255)
     date_created = models.DateTimeField(auto_now_add=True)
 
     def __unicode__(self):
@@ -184,7 +190,10 @@ class MultiImage(AbstractPromotion):
     """
     _type = 'Multi-image'
     name = models.CharField(_("Name"), max_length=128)
-    images = models.ManyToManyField('promotions.Image', null=True, blank=True)
+    images = models.ManyToManyField(
+        'promotions.Image', null=True, blank=True,
+        help_text=("Choose the Image content blocks that this block will use. "
+                   "(You may need to create some first)."))
     date_created = models.DateTimeField(auto_now_add=True)
 
     def __unicode__(self):
@@ -199,7 +208,7 @@ class SingleProduct(AbstractPromotion):
     _type = 'Single product'
     name = models.CharField(_("Name"), max_length=128)
     product = models.ForeignKey('catalogue.Product')
-    description = models.TextField(_("Description"), null=True, blank=True)
+    description = models.TextField(_("Description"), blank=True)
     date_created = models.DateTimeField(auto_now_add=True)
 
     def __unicode__(self):
@@ -219,9 +228,9 @@ class AbstractProductList(AbstractPromotion):
     of products.
     """
     name = models.CharField(_("Title"), max_length=255)
-    description = models.TextField(_("Description"), null=True, blank=True)
+    description = models.TextField(_("Description"), blank=True)
     link_url = ExtendedURLField(_('Link URL'), blank=True, null=True)
-    link_text = models.CharField(_("Link text"), max_length=255, blank=True, null=True)
+    link_text = models.CharField(_("Link text"), max_length=255, blank=True)
     date_created = models.DateTimeField(auto_now_add=True)
 
     class Meta:
@@ -242,11 +251,13 @@ class HandPickedProductList(AbstractProductList):
     products.
     """
     _type = 'Product list'
-    products = models.ManyToManyField('catalogue.Product', through='OrderedProduct', blank=True, null=True,
-        verbose_name=_("Products"))
+    products = models.ManyToManyField('catalogue.Product',
+                                      through='OrderedProduct', blank=True,
+                                      null=True, verbose_name=_("Products"))
 
     def get_queryset(self):
-        return self.products.all().order_by('%s.display_order' % OrderedProduct._meta.db_table)
+        return self.products.base_queryset()\
+            .order_by('%s.display_order' % OrderedProduct._meta.db_table)
 
     def get_products(self):
         return self.get_queryset()
@@ -258,7 +269,8 @@ class HandPickedProductList(AbstractProductList):
 
 class OrderedProduct(models.Model):
 
-    list = models.ForeignKey('promotions.HandPickedProductList', verbose_name=_("List"))
+    list = models.ForeignKey('promotions.HandPickedProductList',
+                             verbose_name=_("List"))
     product = models.ForeignKey('catalogue.Product', verbose_name=_("Product"))
     display_order = models.PositiveIntegerField(_('Display Order'), default=0)
 
@@ -276,20 +288,17 @@ class AutomaticProductList(AbstractProductList):
         (BESTSELLING, _("Bestselling products")),
         (RECENTLY_ADDED, _("Recently added products")),
     )
-    method = models.CharField(_('Method'), max_length=128, choices=METHOD_CHOICES)
-    num_products = models.PositiveSmallIntegerField(_('Number of Products'), default=4)
+    method = models.CharField(_('Method'), max_length=128,
+                              choices=METHOD_CHOICES)
+    num_products = models.PositiveSmallIntegerField(_('Number of Products'),
+                                                    default=4)
 
     def get_queryset(self):
         Product = get_model('catalogue', 'Product')
+        qs = Product.browsable.base_queryset()
         if self.method == self.BESTSELLING:
-            return (Product.browsable.all()
-                    .select_related('stockrecord__partner')
-                    .prefetch_related('variants', 'images')
-                    .order_by('-score'))
-        return (Product.browsable.all()
-                .select_related('stockrecord__partner')
-                .prefetch_related('variants', 'images')
-                .order_by('-date_created'))
+            return qs.order_by('-score')
+        return qs.order_by('-date_created')
 
     def get_products(self):
         return self.get_queryset()[:self.num_products]
@@ -301,7 +310,8 @@ class AutomaticProductList(AbstractProductList):
 
 class OrderedProductList(HandPickedProductList):
     tabbed_block = models.ForeignKey('promotions.TabbedBlock',
-                                     related_name='tabs', verbose_name=_("Tabbed Block"))
+                                     related_name='tabs',
+                                     verbose_name=_("Tabbed Block"))
     display_order = models.PositiveIntegerField(_('Display Order'), default=0)
 
     class Meta:
