@@ -20,7 +20,11 @@ class Command(BaseCommand):
         except Order.DoesNotExist:
             raise CommandError("No order found with number %s" % args[1])
 
+        ctx = {
+            'order': order,
+            'lines': order.lines.all(),
+        }
         messages = CommunicationEventType.objects.get_and_render(
-            args[0], {'order': order})
+            args[0], ctx)
         print "Subject: %s\nBody:\n\n%s\nBody HTML:\n\n%s" % (
             messages['subject'], messages['body'], messages['html'])
