@@ -7,7 +7,8 @@ def profile(fn):
     """
     Profile the decorated function, storing the profile output in /tmp
 
-    Inspired by https://speakerdeck.com/rwarren/a-brief-intro-to-profiling-in-python
+    Inspired by
+    https://speakerdeck.com/rwarren/a-brief-intro-to-profiling-in-python
     """
     def profiled_fn(*args, **kwargs):
         filepath = "/tmp/%s.profile" % fn.__name__
