@@ -1,5 +1,5 @@
 from django import template
-from django.template.loader import render_to_string, select_template
+from django.template.loader import select_template
 
 
 register = template.Library()
@@ -15,7 +15,8 @@ def render_product(context, product):
     be used for different product classes.
     """
     names = ['catalogue/partials/product/upc-%s.html' % product.upc,
-             'catalogue/partials/product/class-%s.html' % product.product_class.name.lower(),
+             'catalogue/partials/product/class-%s.html'
+             % product.get_product_class().slug,
              'catalogue/partials/product.html']
     template_ = select_template(names)
     # Ensure the passed product is in the context as 'product'
